from commons.core.sql.TableMapAdaptator import TableMapAdaptator
from commons.core.coord.MapUtils import MapUtils
from PASTEC.utils.AbstractAnalyseAgent import AbstractAnalyseAgent
from PASTEC.Demand import Demand
from PASTEC.ORF import ORF
from PASTEC.DemandAndResult import DemandAndResult

    
## This agent search in the database type of ORF in TE consensus and type of profiles in the ORF
#
class ORF_Agent(AbstractAnalyseAgent):

    SUFFIX_TABLE_NAME_LIST = ["_ORF_map"]
    PROFILE_AGENT = "TE_Profiles_Agent"
    CATEGORY_LENGTH_ORF = {
                           0: ">1000bps",
                           1: ">3000bps"}

    def __init__(self, env):
        AbstractAnalyseAgent.__init__(self, env)
        self._listOfKnownAgents = [self.PROFILE_AGENT]

    def analyse(self, demand, lMap):
        consensusName = demand.consensusName
        lMapSorted = MapUtils.getMapListSortedByIncreasingMinThenMax( lMap )
        lProfilesDict, dProfilesFrames = self.askForProfilesInfo(consensusName)
        iDemandAndResult = DemandAndResult(demand)
        iDemandAndResult.result = []
        for iMap in lMapSorted:
            tabTmp = iMap.name.split("|")
            oRFLength = tabTmp[-1]
            frameORF = tabTmp[-2]
            iORF = ORF()
            TESignifCategoryORFBounds = self._environment.getTESignifCategoryORFBounds()
            for size in TESignifCategoryORFBounds:
                if int(oRFLength) > size and int(oRFLength) < TESignifCategoryORFBounds[TESignifCategoryORFBounds.index(size) + 1]:
                    iORF.setSizeCategory(self.CATEGORY_LENGTH_ORF[TESignifCategoryORFBounds.index(size)])
                    lProfileTypes = self._createOrfDescriptionWithProfilesInfo(dProfilesFrames, iMap, frameORF, lProfilesDict)
                    iORF.setProfileTypesList(lProfileTypes)
            if not iORF.isEmpty():
                iDemandAndResult.result.append(iORF)
        return iDemandAndResult
    
    ## To have information about profiles in the consensus
    #
    # @param consensusName string name of the consensus
    # @return lProfilesResultsSummary list of profiles dicts
    # @return dProfilesFrames dict of frames (key = profile name)
    #
    def askForProfilesInfo(self, consensusName):
        lProfilesDict = []
        dProfilesFrames = {}
        iProfilesDemand = Demand(self.PROFILE_AGENT, consensusName)
        if self._environment.isDemandOnBlackBoardResults(iProfilesDemand):
#            iProfilesDemandResult = self._environment.getResultAccordingToDemand(iProfilesDemand)
            iDemandAndResult = self._environment.getResultAccordingToDemand(iProfilesDemand)
            for iProteinDomainMatch in iDemandAndResult.result:
                lProfilesDict.append({"name": iProteinDomainMatch.getName(),
                                          "coverage": float(iProteinDomainMatch.getCoverage()),
                                          "start": iProteinDomainMatch.getStart(),
                                          "type": iProteinDomainMatch.getType(),
                                          "strand": iProteinDomainMatch.getStrand()
                                          })
                dProfilesFrames[iProteinDomainMatch.getName()] = iProteinDomainMatch.getFrame()
#            lProfilesDict = iProfilesDemandResult.listProfilesDetailed
#            dProfilesFrames = iProfilesDemandResult.profilesFrames
        return lProfilesDict, dProfilesFrames
    
    def getListOfFeaturesToAnalyse(self, consensusName):
        iTMA = TableMapAdaptator(self._db, self._tableName)
        lMap = iTMA.getMapListFromChr(consensusName)
        return lMap
    
    def _createOrfDescriptionWithProfilesInfo(self, dProfilesFrames, iMap, frameORF, lProfilesDict):
        lProfileTypes = []
        for resultsItem in lProfilesDict:
            if resultsItem['start'] > iMap.start and resultsItem['start'] <= iMap.end \
                and dProfilesFrames.has_key(resultsItem['name']) and dProfilesFrames[resultsItem['name']] == int(frameORF):
                tabTmp = resultsItem['name'].split("_")
                profileType = tabTmp[-2]
                lProfileTypes.append(profileType)
        return lProfileTypes