from PASTEC.Demand import Demand
from PASTEC.ClassifierResult import ClassifierResult

class AbstractClassifierAdaptatorAgent(object):

    ## Constructor
    #
    # @param env Environment instance
    #
    def __init__(self, env):
        self._environment = env
        self._CI = 0
        self.reset()
        self._setAgentDecisionRules()
    
    def getConclusionsSearch(self):
        return self._conclusionsSearch
    
    def getConclusionsMatch(self):
        return self._conclusionsMatch
    
    def setConclusionsSearch(self, conclusionsSearch):
        self._conclusionsSearch = conclusionsSearch

    def setConclusionsMatch(self, conclusionsMatch):
        self._conclusionsMatch = conclusionsMatch
    
    def setConsensus(self, consensusName):
        self._consensusName = consensusName 
    
    def setDependentAgentsList(self, lAgents):
        self._listOfKnownAgents = lAgents
    
    def getDependentAgentsList(self):
        return self._listOfKnownAgents
       
    def _setAgentDecisionRules(self):
        """sets the current agen'ts decision rules based on info from configuration file"""
        decisionRulesData = self._environment.getAgentDecisionRules(self.__class__.__name__)
        #self.CI_MAX = decisionRulesData["CI_MAX"]
        #self.INIT_CLASSIF = decisionRulesData["INIT_CLASSIF"]
        #self.PATTERN_2_SEARCH_IN_BLAST_RESULTS = decisionRulesData["PATTERN_2_SEARCH_IN_BLAST_RESULTS"] 
        #self.CONDITIONS = decisionRulesData["CONDITIONS"] 
        self.__dict__.update( decisionRulesData ) #update /add all attributes to current instance
        
    ## Get and filter analysis if corresponding table is in database.
    #
    def getAnalysesResults(self, demand):
        for agentName in self._listOfKnownAgents:
            iDemand = Demand(agentName, demand.consensusName)
            iAgentResult = self._environment.getResultAccordingToDemand(iDemand)
            method2Call = "self.makeAppropriateConclusionsFor%s( iAgentResult )" % agentName
            exec ( method2Call )
            
    ## Three main step : make demands, get results and decide
    #
    def live(self):
        for consensusName in self._environment.getConsensusNameList():
            iDemand = Demand(self.__class__.__name__, consensusName)
            self._consensusName = iDemand.consensusName 
            self.reset()

            self.getAnalysesResults(iDemand)
            classification, normalizedIC = self.decide()
            message = self.generateMessage(classification, normalizedIC)
            
            finalResult = ClassifierResult(iDemand, message)
            self._environment.addResultToBlackBoard(finalResult)
                    
    def _normalizeIC(self):
        normalizedIC = 0
        if not self._CI <= 0:
            normalizedIC = (self._CI * self._environment.getMaxCI()) / self.CI_MAX
            if normalizedIC > self._environment.getMaxCI():
                normalizedIC = self._environment.getMaxCI()
        return normalizedIC
    
    def reset(self):
        self._conclusionsSearch = []
        self._conclusionsMatch = []
    
    def decide(self):
        pass
    
    def generateMessage(self, classification, IC):
        pass
    
    def _fillDictOfEvidence(self):
        pass