#! /usr/bin/env python

import tarfile, re, os
from optparse import OptionParser

class Deseq(object):
    
    def __init__(self):
        self._tarFile = ""
        self._header = 0
        self._replicates = 0
        self._html = ""
        self._htmldir = ""
        self._complete = ""
        self._up = ""
        self._down = ""
    
    def setAttributesFromCmdLine(self):
        parser = OptionParser(description = "deseq wrapper", version = "v1.0")
        parser.add_option( "-t", "--tar", dest = "tarFile", help = "tar file name")
        parser.add_option( "-e", "--header", dest = "header", action = "store_const", const = 1, default = 0, help = "with or without header")
        parser.add_option( "-r", "--replicates", dest = "replicates", action = "store_const", const = 1, default = 0, help = "with or without replicates")
        parser.add_option( "-w", "--html", dest = "html", help = "html file")
        parser.add_option( "-d", "--htmldir", dest = "htmldir", help = "dir for Galaxy")
        parser.add_option( "-c", "--complete", dest = "complete", help = "complete xls fileName")
        parser.add_option( "-u", "--up", dest = "up", help = "up xls fileName")
        parser.add_option( "-b", "--down", dest = "down", help = "down xls fileName")
        options, args = parser.parse_args()
        self._setAttributesFromOptions(options)

    def _setAttributesFromOptions(self, options):
        self._tarFile = options.tarFile
        self._header = options.header
        self._replicates = options.replicates
        self._html = options.html
        self._htmldir = options.htmldir
        self._complete = options.complete
        self._up = options.up
        self._down = options.down
    
    def run(self):
        tar = tarfile.open(self._tarFile, "r")
        outputName = "list.tmp"
        out = open(outputName, "w")
        out.write("label\tfiles\tgroup\n")
        for tarinfo in tar:    
            name = tarinfo.name
            label = re.match("Group\d{1}_\d{1}", os.path.split(name)[1]).group(0)
            group = re.search("\d{1}", label).group(0)
            out.write(label + '\t' + name + '\t' + group + '\n')
            tar.extract(name)
	tar.close()
        out.close()
        
        os.makedirs(self._htmldir)
        scriptPath = os.path.abspath(__file__)
        path = os.path.dirname(scriptPath)
        os.system("cat %s/DESeqTools/anadiffGenes2conds.R | R --slave --args %s %s %s %s %s %s %s %s %s < %s/DESeqTools/anadiffGenes2conds.R" %(path, outputName, self._header, self._replicates, self._html, self._htmldir, self._complete, self._up, self._down, scriptPath, path))
    
if __name__ == "__main__":
    deseq = Deseq()
    deseq.setAttributesFromCmdLine()
    deseq.run()

