from MySQLdb import DataError

RETRIEVERS = ["Profiles_Agent", "TE_Profiles_Agent", "TRSearch_Agent", "TE_BLRn_Agent", \
              "HG_BLRn_Agent", "rDNA_BLRn_Agent", "TE_BLRtx_Agent", "TE_BLRx_Agent", \
              "SSRtrf_Agent", "PolyA_Agent", "Size_Agent", "ORF_Agent", "Strand_Agent"]

CLASSIFIERS = ["LINE_Agent", "Helitron_Agent", "rDNAAdaptator_Agent", "HG_BLRnAdaptatorAgent", \
               "HG_profilesAdaptatorAgent", "SSRAdaptator_Agent", "LTR_Agent", "PLE_Agent", \
               "DIRS_Agent", "SINE_Agent", "LARD_Agent", "TRIM_Agent", "TIR_Agent", "Maverick_Agent", \
               "Crypton_Agent", "MITE_Agent", "ClassI_Agent", "ClassII_Agent"]

## Use this class to create a agent instance
#
class AgentFactory(object):
    
    @staticmethod
    def getClassifierNamesList():
        return CLASSIFIERS
    
    @staticmethod
    def createAgent(agentName, env):
        iAgent = None
        exec("from PASTEC.%s import %s" % (agentName, agentName))
        exec("iAgent = %s(env)" % agentName)
        return iAgent
    
    @staticmethod
    def createRetriever(env, agentName, lRetrieversAlreadyCreated = []):
        iAgent = AgentFactory.createAgent(agentName, env)
        
        lTableSuffix = iAgent.getSuffixTableNameList()
        for tableSuffix in lTableSuffix:
            tableName = env.getProjectName() + tableSuffix
            if tableName not in env.getTablesList():
                raise DataError("Corresponding SQL Table doesn't exist")
            
        AgentFactory._setDependentAgentListOnIAgent(iAgent, lRetrieversAlreadyCreated)
        return iAgent

    @staticmethod
    def createClassifier(env, agentName, lRetrieversAlreadyCreated = []):
        iAgent = AgentFactory.createAgent(agentName, env)
        AgentFactory._setDependentAgentListOnIAgent(iAgent, lRetrieversAlreadyCreated)            
        return iAgent

    @staticmethod
    def _setDependentAgentListOnIAgent(iAgent, lAllAgents):
        sAllAgents = set(lAllAgents)
        sDependentAgents = set(iAgent.getDependentAgentsList())
        sCreatedDependentAgents = sAllAgents.intersection(sDependentAgents)
        iAgent.setDependentAgentsList(list(sCreatedDependentAgents))
        
    @staticmethod
    def generateRetrieverResultsAndGetRetrieverNames(env):
        lCreatedRetrieverNames = []
        for agentName in RETRIEVERS:
            try:
                iAgent = AgentFactory.createRetriever(env, agentName, lCreatedRetrieverNames)
                iAgent.live()
                lCreatedRetrieverNames.append(iAgent.__class__.__name__)
            except DataError:
                pass
        return lCreatedRetrieverNames
    
    @staticmethod
    def generateClassifierResults(env, lCreatedRetrieverNames):
        for agentName in CLASSIFIERS:
            iAgent = AgentFactory.createClassifier(env, agentName, lCreatedRetrieverNames)
            iAgent.live()
