## This class represents a protein domain match on a sequence
#
class ProteinDomainMatch(object):
    # Constructor
    #
    # @param name string Profile name
    # @param coverage float Profile proportion presents in the sequence (percentage)
    # @param start integer Position on query sequence corresponding with the beginning of profile
    # @param strand string Value can be "+", "-", "."
    # @param frame integer Value can be 1, 2, 3, -1, -2, -3  
    #
    def __init__(self, name, coverage = 0.0, start = 0, strand = "", frame = 0, end = 0, index = 0):
        self._name = name
        self._coverage = coverage
        self._start = start
        self._strand = strand
        self._frame = frame
        self._end = end
        self._index = index
        
    def __eq__(self, o):
        return self._name == o._name and round(self._coverage, 3) == round(o._coverage, 3) \
            and self._start == o._start and self._strand == o._strand \
            and self._frame == o._frame and self._end == o._end and self._index == o._index
            
    def getType(self):
        return self._name.split("_")[-2]
    
    def getName(self):
        return self._name
    
    def getStart(self):
        return self._start
    
    def getCoverage(self):
        return float(self._coverage)
    
    def getFrame(self):
        return self._frame
    
    def getStrand(self):
        return self._strand
    
    def setEnd(self, end):
        self._end = end
        
    def setStrand(self, strand):
        self._strand = strand
        
    def increaseCoverage(self, coverage):
        self._coverage += coverage
        
    def computeIndex(self, scoreWeight):
        self._index = round(self._coverage * scoreWeight, 2)