#!/usr/bin/env python

import os
from commons.core.parsing.FastaParser import FastaParser
from commons.core.utils.FileUtils import FileUtils
from commons.core.utils.RepetOptionParser import RepetOptionParser

class ReverseComplementAccordingToClassif(object):
    
    def __init__(self, fastaFile = None, classifFileName = None, isOutClassif = False):
        self._fastaFile = fastaFile
        self._classifFileName = classifFileName
        self._isOutClassif = isOutClassif
        
    def setFastaFile(self, fastaFile):
        self._fastaFile = fastaFile
        
    def setClassifFile(self, classifFile):
        self._classifFileName = classifFile
        
    def setIsOutClassif(self, isOutClassif):
        self._isOutClassif = isOutClassif
    
    def setAttributesFromCmdLine(self):
        parser = RepetOptionParser(version = "ReverseComplementAccordingToClassif.py v1.0")
        parser.add_option('-f', '--fasta', dest='fastaFile', help='fasta file', default= None)
        parser.add_option('-i', '--input', dest='classifFileName', help='classification file from PASTEC', default= None)
        parser.add_option('-n', '--new',   dest='isOutClassif', help='demand output classification file with reversed headers', default= False, action="store_true")
        (options, args) = parser.parse_args()
        self._setAttributesFromOptions(options)
        
    def _setAttributesFromOptions(self, options):
        self.setFastaFile(options.fastaFile)
        self.setClassifFile(options.classifFileName)
        self.setIsOutClassif(options.isOutClassif)
        
    def checkOptions(self):        
        if self._fastaFile == None:
            raise Exception("ERROR: No fasta file specified in option!")
        elif not FileUtils.isRessourceExists(self._fastaFile):
            raise Exception("ERROR: Can't open fasta file at %s" % self._fastaFile)
        
        if self._classifFileName == None:
            raise Exception("ERROR: No classification file specified in option!")
        elif not FileUtils.isRessourceExists(self._classifFileName):
            raise Exception("ERROR: Can't open classification file at %s" % self._classifFileName)
    
    def _writeOutputClassif(self, dNameToNegStrand):
        outClassifFile = open("%s_reversed" % self._classifFileName, 'a')
        classifFile = open(self._classifFileName, "r")
        line = classifFile.readline()
        while line:
            newline = line
            if line.split('\t')[0] in dNameToNegStrand.keys():
                newName = ["%s_reversed" % line.split('\t')[0]]
                newName.extend(line.split('\t')[1:])
                newline = '\t'.join(newName)
            outClassifFile.write(newline)
            line = classifFile.readline()
        classifFile.close()
        outClassifFile.close()
    
    def run(self):
        self.checkOptions()
        classifFile = open(self._classifFileName, "r")
        dNameToNegStrand = {}
        line = classifFile.readline()
        while line:
            if line.split('\t')[2] == '-':
                dNameToNegStrand[line.split('\t')[0]] = '-'
            line = classifFile.readline()
        classifFile.close()
        
        fileName = os.path.basename(self._fastaFile)
        fileWithoutExt = os.path.splitext(fileName)[0]
        resultFile = open("%s_negStrandReversed.fa" % fileWithoutExt, "w")
        iFastaParser = FastaParser(self._fastaFile)
        for iSequence in iFastaParser.getIterator():
            if iSequence.getName() in dNameToNegStrand.keys():
                iSequence.reverseComplement()
                iSequence.setName("%s_reversed" % iSequence.getName())
            resultFile.write(iSequence.printFasta())
        resultFile.close()
        
        if self._isOutClassif:
            self._writeOutputClassif(dNameToNegStrand)
        
if __name__ == '__main__':
    iReverseComplementAccordingToClassif = ReverseComplementAccordingToClassif()
    iReverseComplementAccordingToClassif.setAttributesFromCmdLine()
    iReverseComplementAccordingToClassif.run()