import glob, os, shutil, re, time, sys, fnmatch
from distutils.core import setup
from distutils.core import Command
from devTools.SetupUtils import SetupUtils

LOG_DEPTH = "setup"
WAIT_INTERVAL = 5
disableEmails = ""
resources = "-q 'test=TRUE'"

verbose = 1

def superGlob(pattern):
    parts = pattern.split("/")
    directory = parts[0]
    pattern = parts[1]
    results = []
    for root, dirs, files in os.walk(directory):
        for basename in files:
            if fnmatch.fnmatch(basename, pattern):
                filename = os.path.join(root, basename)
                if os.path.splitext(filename)[1] not in [".py", ".pyc"]:
                    results.append(filename)
    return results


class Test(Command):
    description = "Run build tests"
    user_options = []
 
    def initialize_options(self):
        """Use this to set option defaults before parsing."""
        pass
 
    def finalize_options(self):
        """Code to validate/modify command-line/config input goes here."""
        pass

    def run(self):
        if verbose > 0:
            print "START Testing (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        
        if verbose > 0:
            print "Cleaning '%s' database" % os.environ["REPET_DB"]
            sys.stdout.flush()
        os.system("ListAndDropTables.py -d '*'")
        
        self._folderToTest = os.environ["REPET_PATH"]
        self._folderToTest = os.path.basename(self._folderToTest.rstrip('/'))
        self._folderToTestBackuped = "%s_backup" % self._folderToTest
           
        if verbose > 0:
            print "Creating source directory backup (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        shutil.copytree(self._folderToTest, self._folderToTestBackuped, symlinks = True)
       
        self.launchUnitaryAndAcceptanceTests()
        self.cleanArbo()
        self.launchAllFunctionalTests()
        
        if verbose > 0:
            print "END Testing (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
            
    def cleanArbo(self):
        if verbose > 0:
            print "START Cleaning source directory (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        
        shutil.rmtree(self._folderToTest)
        shutil.move(self._folderToTestBackuped, self._folderToTest)
        
        splittingPattern = os.path.basename(self._folderToTest)
        lTestOrDataDirs = [i for i in SetupUtils.find_dirs(self._folderToTest) if SetupUtils.isTestOrDataDir(i, splittingPattern)]
        SetupUtils.moveTree(lTestOrDataDirs, "_movedFiles", splittingPattern)
        
        if verbose > 0:
            print "END Cleaning source directory (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
            
    def launchUnitaryAndAcceptanceTests(self):
        if verbose > 0:
            print "Launching unitary tests build for SMART (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        os.system("Build.py -c unitary %s -b -d -m -v 1 %s -r %s -p SMART,commons/core/writer,commons/core/parsing -t 'Release SMART' -g 'release' > unitary_tests_SMART.log 2>&1 &" % (resources, disableEmails, self._folderToTest))
        time.sleep(WAIT_INTERVAL)
        
        if verbose > 0:
            print "Launching acceptance tests build for SMART (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        os.system("Build.py -c acceptance %s -b -d -m -v 1 %s -r %s -p SMART,commons/core/writer,commons/core/parsing -t 'Release SMART' -g 'release' > acceptance_tests_SMART.log 2>&1 &" % (resources, disableEmails, self._folderToTest))
        time.sleep(WAIT_INTERVAL)
        
        if verbose > 0:
            print "Launching unitary tests build for PASTEC (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        os.system("Build.py -c unitary %s -b -d -m -v 1 %s -r %s -p PASTEC -t 'Release PASTEC' -g 'release' > unitary_tests_PASTEC.log 2>&1 &" % (resources, disableEmails, self._folderToTest))
        time.sleep(WAIT_INTERVAL)
        
        if verbose > 0:
            print "Launching acceptance tests build for PASTEC (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        os.system("Build.py -c acceptance %s -b -d -m -v 1 %s -r %s -p PASTEC -t 'Release PASTEC' -g 'release' > acceptance_tests_PASTEC.log 2>&1 &" % (resources, disableEmails, self._folderToTest))
        time.sleep(WAIT_INTERVAL)

        if verbose > 0:
            print "Launching unitary tests build for Commons (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        os.system("Build.py -c unitary %s -b -d -m -v 1 %s -r %s -p commons -t 'Release  Commons' -g 'release' > unitary_tests_commons.log 2>&1 &" % (resources, disableEmails, self._folderToTest))
        time.sleep(WAIT_INTERVAL)
        
        if verbose > 0:
            print "Launching acceptance tests build for Commons (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        os.system("Build.py -c acceptance %s -b -d -m -v 1 %s -r %s -p commons -t 'Release Commons' -g 'release' > acceptance_tests_commons.log 2>&1" % (resources, disableEmails, self._folderToTest))
        
    def launchAllFunctionalTests(self):
        TMP_PATH = "%s/tests" %  os.getcwd()
        if os.path.exists(TMP_PATH):
            shutil.rmtree(TMP_PATH)
        os.makedirs(TMP_PATH)
        
        if verbose > 0:
            print "Launching functional tests build (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        os.system("Build.py -c functional %s -b -d -m -v 1 -w %s %s -r %s -t 'Release PASTEClassifier' -g 'release' > functional_tests.log 2>&1" % (resources, TMP_PATH, disableEmails, self._folderToTest))
        time.sleep(WAIT_INTERVAL)


class PublicRelease(Command):
    description = "remove test directory and TE_finder folder in the tree files for public release"
    user_options = []
 
    def initialize_options(self):
        """Use this to set option defaults before parsing."""
        pass
 
    def finalize_options(self):
        """Code to validate/modify command-line/config input goes here."""
        pass
 
    def run(self):
        if verbose > 0:
            print "START Releasing PASTEClassifier! (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()

        cwd = os.getcwd()
        if verbose > 0:
            print "Removing all '.pyc' files and dead symlinks in bin, then adding execution rights to all 'py' files (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        os.system("find . -name '*.pyc' -exec rm {} \;")
        os.system("find . -name '*.py' -exec chmod +x {} \;")
        os.system("find -L ./bin -type l -exec rm {} +")
        os.chdir("..")
        os.system("tar -czf %s.tar.gz %s" % (os.path.basename(cwd), os.path.basename(cwd)))

        if verbose > 0:
            print "END Releasing PASTEClassifier! (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()

class Compile(Command):
    description = "compile a project"
    user_options = []
 
    def initialize_options(self):
        """Use this to set option defaults before parsing."""
        pass
 
    def finalize_options(self):
        """Code to validate/modify command-line/config input goes here."""
        pass
 
    def run(self):
        if verbose > 0:
            print "START Compiling (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
            
        os.system("cd TE_finder; make; make install; cd ../")
        
        if verbose > 0:
            print "END Compiling (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
    
class Install(Command):
    description = "Install a project"
    user_options = []
 
    def initialize_options(self):
        """Use this to set option defaults before parsing."""
        pass
 
    def finalize_options(self):
        """Code to validate/modify command-line/config input goes here."""
        pass

    def _isToLink(self, fileName):
        if re.search("__\w*.py", fileName):
            return False
        elif re.search("^Test\w*.py", fileName):
            return False
        elif re.search("\w*.py$", fileName):
            return True
        return False        

    def run(self):
        cwd = os.getcwd()
        print "Building PASTEClassifier in %s" % (cwd)
        
        if verbose > 0:
            print "START Installing (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        
        if os.path.isdir("bin"):
            shutil.rmtree("bin")
        os.mkdir("bin")
        os.chdir("bin")
        lPackages = ["commons", "PASTEC"]
        for package in lPackages:
            if os.path.isdir("../%s" % package):
                print "processing %s/..." % package
                [os.symlink(i, os.path.basename(i)) for i in SetupUtils.findall("../%s" % package) if self._isToLink(os.path.basename(i))]
        os.system("chmod 0755 *")
        if os.path.isdir("../TE_finder"):
            print "processing TE_finder/..." 
            [shutil.copy(i, os.path.basename(i)) for i in SetupUtils.findall("../TE_finder/bin")]
            os.symlink("blaster2.25", "blaster")
            os.symlink("grouper2.25", "grouper")
            os.symlink("matcher2.25", "matcher")
            
        shutil.rmtree("../TE_finder")
            
        if verbose > 0:
            print "FINISHED Installing (%s)" % time.strftime("%Y-%m-%d %H:%M:%S")
            sys.stdout.flush()
        print "PASTEClassifier is ready to use"
    
setup(
      name = "PASTEClassifier",
      version = "1.0",
      description = 'The PASTEClassifier package to classify a DNA repeats library.',
      author = 'REPET Team',
      author_email = 'urgi-repet@inra.versailles.fr',
      url = 'http://urgi.versailles.inra.fr/Tools/REPET',
      long_description = "",
      license = "CeCILL http://www.cecill.info",
      packages = SetupUtils.find_sub_packages(["commons", "PASTEC", "pyRepet", "SMART"]),
      #Additionnal Files (not .py)
      data_files = [('PASTEC/doc', glob.glob('PASTEC/doc/*')),('PASTEC', superGlob('PASTEC/*')), ('SMART/Java/Python', SetupUtils.findall('SMART/Java/Python')),
                    ('SMART/data', SetupUtils.findall('SMART/data')), ('commons', superGlob('commons/*')),
                    ('config', ["config/setEnv.sh","config/PASTEClassifier.cfg","config/PASTEClassifier_parallelized.cfg"]),
                    ('TE_finder', superGlob("TE_finder/*")), ('',['LICENSE'])],
      cmdclass = {
                  'compile': Compile, 'install' : Install, 'test': Test, 'release': PublicRelease
                 }
      )